#!/opt/puppetlabs/puppet/bin/ruby
#
#
# See the file "LICENSE" for the full license governing this code.
#
#
require 'json'
require 'open3'
require 'puppet'
begin
  Puppet.initialize_settings
rescue Puppet::DevError
  # When run in Rspec, this is called multiple times
end
require 'easy_type'
require 'puppet_x/enterprisemodules/weblogic/core'

class StopManagedServersTask

  STOP_STATES  = [
    'SHUTTING-DOWN',
    'SUSPENING',
    'FORCE_SUSPEND',
    'FAILED_NOT_RESTARTABLE',
    'SHUTDOWN'
  ]

  def self.run
    instance = self.new
    instance.run
  end

  def run()
    begin
      fetch_input()
      execute()
      sync()
    rescue
      raise
    ensure
      report()
    end
  end

  def fetch_input
    params  = JSON.parse(STDIN.read)
    @domain  = params.fetch('domain')       { 'default'}
    @skip    = params.fetch('skip')         { []}
    @wait    = params.fetch('wait')         { true}.to_s == 'true'
    @timeout = params.fetch('timeout')      { 300 }.to_i
    @between = params.fetch('between_time') { 10}.to_i
    @result  = {}
  end

  def report()
    puts @result.to_json
  end

  def execute
    begin
      @wls_config = Puppet_X::EnterpriseModules::WebLogic::Tree.new(@domain)
      begin
        # Check if the domain exists
        @wls_config.validate
      rescue RuntimeError => e
        @result[:_error] = { 
          msg: e.message,
          kind: "enterprisemodules-wls_config/admin_server_not_running"
        }
        raise e
      rescue ArgumentError => e
        @result[:_error] = { 
          msg: e.message,
          kind: "enterprisemodules-wls_config/unknown_domain"
        }
        raise e
      end
      admin_server = @wls_config.admin_server
      all_servers  = @wls_config.managed_servers
      @servers      = all_servers - [admin_server] - @skip
      @servers.each do | server|
        state = @wls_config.state_for(server)
        if STOP_STATES.include?(state)
          @result['already_stopped_managed_servers'] ||= []
          @result['already_stopped_managed_servers'] << server
          next
        end
        @wls_config.stop_server(server)
        @result['stopped_managed_servers'] ||= []
        @result['stopped_managed_servers'] << server
      end
      @result['skipped'] = @skip if @skip.any?
    end
  end

  def sync
    if @wait
      begin
        Timeout::timeout(@timeout) do
          loop do
            states = @servers.collect { |server| @wls_config.state_for(server)}
            break if states.all?{|s| s == 'SHUTDOWN'}
            sleep(@between)
          end
        end
      rescue => e
        @result[:_error] = { 
          msg: e.message,
          kind: "enterprisemodules-wls_config/timeout"
        }
        raise e
      end
    end
  end


end

StopManagedServersTask.run if __FILE__ == $0